package com.isu.gaswellwatch.modbus.data.impl;

import cn.hutool.core.map.MapUtil;
import com.isu.gaswellwatch.config.SnowflakeConfig;
import com.isu.gaswellwatch.modbus.data.PersistenceHandler;
import jakarta.annotation.Resource;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.dao.DataAccessException;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCallback;
import org.springframework.stereotype.Component;
import org.springframework.util.ResourceUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Map;
import java.util.Objects;

/**
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2024/11/24 17:33
 */
@Component(PersistenceHandler.MODBUS_DEVICE_TYPE)
public class Knpcv1PersistenceHandler implements PersistenceHandler {

    @Resource
    private JdbcTemplate jdbcTemplate;
    @Resource
    private SnowflakeConfig snowflakeConfig;
    @Resource(name = "redisTemplate")
    private RedisTemplate redisTemplate;

    @Override
    public void createTable(String tableName) {
        String createTableSQL = getResource("sql/CREATE_KNPCV1.sql");
        this.jdbcTemplate.execute(StringUtils.replace(createTableSQL, "$TableName$", tableName));
    }

    @Override
    public void insert(String tableName, String cacheKey) {
        String insertTableSQL = getResource("sql/INSERT_KNPCV1.sql");
        insertTableSQL = StringUtils.replace(insertTableSQL, "$TableName$", tableName);

        Map<String, Object> row = this.redisTemplate.opsForHash().entries(cacheKey);

        this.jdbcTemplate.execute(insertTableSQL, new PreparedStatementCallback<Object>() {
            @Override
            public Object doInPreparedStatement(PreparedStatement ps) throws SQLException, DataAccessException {
                ps.setLong(1, Knpcv1PersistenceHandler.this.snowflakeConfig.snowflakeId());
                Knpcv1PersistenceHandler.this.setValue(ps, row, 2, "deviceId", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 3, "collectionTime", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 4, "receiveTime", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 5, "status", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 6, "device_time", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 7, "runMode", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 8, "wellStatus", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 9, "plugStatus", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 10, "statusOpenTime", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 11, "statusCloseTime", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 12, "oilPressure", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 13, "casPressure", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 14, "prePressure", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 15, "pipePressure", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 16, "liquidLevel", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 17, "temperature", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 18, "humidity", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 19, "opmode", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 20, "well_ctl", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 21, "timer1", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 22, "timer2", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 23, "timer1Open", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 24, "timer1Close", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 25, "timer2Open", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 26, "timer2Close", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 27, "timingOpen", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 28, "timingClose", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 29, "timingCelay", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 30, "presource", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 31, "pressureOpen", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 32, "pressureClose", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 33, "triggerType", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 34, "stabilityTime", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 35, "maxOpenWell", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 36, "maxCloseWell", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 37, "minOpenWell", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 38, "minCloseWell", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 39, "presproTect", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 40, "presproSource", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 41, "openWellLimitMax", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 42, "openWellLimitMin", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 43, "plugInitStatus", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 44, "plugSustainTime", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 45, "plugCloseTime", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 46, "tpInitStatus", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 47, "tpOpenSource", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 48, "tpOpenTrigger", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 49, "tpOpenPressure", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 50, "tpOpenTime", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 51, "tpCloseSource", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 52, "tpCloseTrigger", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 53, "tpClosePressure", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 54, "tpCloseTime", Types.BIGINT);
                Knpcv1PersistenceHandler.this.setValue(ps, row, 55, "tpStabilityTime", Types.BIGINT);
                return ps.executeUpdate();
            }
        });

    }

    public void setValue(PreparedStatement ps, Map<String, Object> row, int index, String key, int sqlType) throws SQLException {
        String value = MapUtil.getStr(row, key);
        if (Objects.isNull(value)) {
            ps.setNull(index, sqlType);
        } else {
            ps.setObject(index, value);
        }
    }

    public static String getResource(String classPath) {
        try {
            File file = ResourceUtils.getFile("classpath:" + classPath);
            return IOUtils.toString(new FileInputStream(file), StandardCharsets.UTF_8);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

}
