package com.isu.gaswellwatch.modbus.data.impl;

import cn.hutool.core.map.MapUtil;
import com.isu.gaswellwatch.modbus.data.PersistenceHandler;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.core.annotation.Order;
import org.springframework.data.redis.core.Cursor;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ScanOptions;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2024/11/23 11:55
 */
@Slf4j
@Component
@SuppressWarnings("all")
public class Redis2DBPersistenceService {

    private static final String EXISTS_TABLE_SQL = "SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA='gaswellwatch' AND TABLE_NAME=?";
    @Resource
    private JdbcTemplate jdbcTemplate;
    @Resource(name = "redisTemplate")
    private RedisTemplate redisTemplate;
    @Resource
    private Map<String, PersistenceHandler> persistenceHandlerMap;
    private RestTemplate restTemplate = new RestTemplate();

    @Order(100)
    @Scheduled(cron = "5/30 * * * * ? ")
    public void write() {
        Map<Long, String> idGatewayMappingMap = null;
        try {
            String onlineGateways = this.restTemplate.getForObject("http://127.0.0.1:9999/modbus-tcp/online", String.class);
            if (StringUtils.isBlank(onlineGateways)) {
                return;
            }
            List<Map<String, Object>> idGatewayMappingList = this.jdbcTemplate
                    .queryForList("select id, gateway_sn from devices where gateway_sn in (" + onlineGateways + ")");
            if (ObjectUtils.isEmpty(idGatewayMappingList)) {
                return;
            }
            idGatewayMappingMap = new HashMap<Long, String>();
            for (Map<String, Object> map : idGatewayMappingList) {
                idGatewayMappingMap.put(MapUtil.getLong(map, "id"), MapUtil.getStr(map, "gateway_sn"));
            }
        } catch (Exception e) {
            log.error("Get online devices list fail", e);
        }

        HashOperations operations = this.redisTemplate.opsForHash();
        try (Cursor<String> cursor = this.redisTemplate.scan(ScanOptions.scanOptions()
                .match(PersistenceHandler.DEVICE_DATA_CACHE + "*").build())) {
            Long deviceId;
            String cacheKey, tableName;
            Map<String, Object> deviceMap, temp;
            PersistenceHandler persistenceHandler;
            while (cursor.hasNext()) {
                deviceMap = operations.entries(cacheKey = cursor.next());
                deviceId = MapUtil.getLong(deviceMap, "deviceId");
                operations.put(PersistenceHandler.DEVICE_DATA_CACHE + deviceId,
                        "online", idGatewayMappingMap.containsKey(deviceId));

                persistenceHandler = persistenceHandlerMap.get("KNPCV1");
                tableName = "t_device_data_" + deviceId;
                temp = this.jdbcTemplate.queryForMap(EXISTS_TABLE_SQL, tableName);  //$TableName$
                if (StringUtils.isBlank(MapUtil.getStr(temp, "TABLE_NAME"))) {
                    persistenceHandler.createTable(tableName);
                }
                persistenceHandler.insert(tableName, cacheKey);
            }
        }
    }

}
